#!/usr/bin/env python
#This source code is in the public domain. It may be used, modified,
#and/or redistributed without restriction.

subjectdistances = range(100, 100000, 10)

def gen_apertures():
  rv = []
  a = 0.7
  rv.append(a)
  while a < 5.6:
    a = a + 0.1
    rv.append(a)
  return rv

apertures = [0.7, 1.4, 2.8, 5.6]
focallengths = [50, 100, 150, 200, 250, 300, 350, 400]
apertures_stepped = gen_apertures()

def calculate_blur(focusdistance, backgrounddistance, aperture, focallength):
  aperturesize = float(focallength) / float(aperture)
  c = aperturesize * \
            ( 
              ( abs(backgrounddistance - focusdistance) ) / backgrounddistance 
            ) 
  coc = c * (
          focallength / (focusdistance - focallength)
      )
  return coc


# first, demonstrate the linear relationship between background blur and
# aperture
fd = open("data/blur-0.7-5.6.dat", "w")
for aperture in apertures_stepped:
  aperturesize = 50 / float(aperture)
  fd.write("%s %s\n" % (aperturesize, calculate_blur(1000.0, 4000.0, aperture, 50.0)))
fd.close()

# now things get interesting. we're going to take the same subject
# distance, but we're going to try it with different focal lengths
# with a fixed ABSOLUTE aperture
fd = open("data/blur-50mm-400mm.dat", "w")
for fl in focallengths:
  aperture = 1.4 * (fl / 50)
  blur = calculate_blur(1000.0, 4000.0, aperture, fl)
  fd.write("%s %s\n" % (fl, blur))

fd.close()

# now we're going to do the above, *but* we're going to keep the
# same framing in all shots. we're also going to try it with
# different background distances: 4, 8, 16 and 32 metres.
# we're also going to use the same figures to generate a file
# using *relative* apertures.

backgrounddistances = [4000.0, 8000.0, 16000.0, 32000.0, 64000.0, 128000.0, 256000.0]
# increase our range of focallengths
focallengths2 = focallengths + [800, 1200, 1600, 2400, 3200]
# we're going to generate it for the < 50mm range too
focallengths_short = range(1, 51)

for bd in backgrounddistances:
  fd = open("data/framing-absoluteaperture/bg-%s.dat" % int(bd), "w")
  fd2 = open("data/framing-relativeaperture/bg-%s.dat" % int(bd), "w")
  fd3 = open("data/framing-relativeaperture/bg-%s-2.dat" % int(bd), "w")
  # fd4: short focal lengths
  fd4 = open("data/framing-relativeaperture/bg-%s-3.dat" % int(bd), "w")
  for fl in focallengths:
    sdistance = 1000.0 * (fl / 50)
    aperture = 1.4 * (fl / 50)
    bd2 = bd + (sdistance - 1000)
    blur = calculate_blur(sdistance, bd2, aperture, fl)
    blur_ra = calculate_blur(sdistance, bd2, 1.4, fl)
    print "f=%s aperture=%s sdistance=%s POI=%s blur=%s" % (fl, 1.4, sdistance, bd2, blur_ra)
    fd.write("%s %s\n" % (fl, blur))
    fd2.write("%s %s\n" % (fl, blur_ra))
  # doing it again, but with an extended range of focal lengths
  for fl in focallengths2:
    sdistance = 1000.0 * (fl / 50)
    bd2 = bd + (sdistance - 1000)
    blur_ra = calculate_blur(sdistance, bd2, 1.4, fl)
    fd3.write("%s %s\n" % (fl, blur_ra))
  for fl in focallengths_short:
    sdistance = 1000.0 * (fl / 50.0)
    bd2 = bd + (sdistance - 1000)
    blur = calculate_blur(sdistance, bd2, 1.4, fl)
    fd4.write("%s %s\n" % (fl, blur))
  fd.close()
  fd2.close()
  fd3.close()
  fd4.close()

aperture = 2.8
frame_sizes = [56.0, 36.0, 24.0, 16.0, 8.0]
focallengths = [28.0] + focallengths
background_distance = 5000.00
for fs in frame_sizes:
  # adjust background distance and subject distance to keep the
  # same framing
  fd = open("data/formats/ra-keepframing-%smm-frame" % int(fs), "w")
  for fl in focallengths:
    sdistance = 1000.0 * (fl / 50)
    bd2 = background_distance + (sdistance - 1000)
    realfl = fl / (36 / fs)
    print "%s on %smm frame = %s real focal length" % (fl, fs, realfl) 
    bcsize = calculate_blur(sdistance, bd2, aperture, realfl)
    bcsize_relative = bcsize * (36.0 / fs)
    bcsize_percentage = (bcsize / fs) * 100.0
    print "%smm on %smm frame: %s blur circle, %s percent" % (
      realfl, fs, bcsize, bcsize_percentage)
    fd.write("%s %s\n" % (fl, bcsize_percentage))
  fd.close()

subject_distances = [1000, 2000, 3000, 4000]


# to examine the effect above, we want to see the size of the
# defocus blur circles of different focal lengths *while*
# maintaining the same subject distances and background 
# distances, with the same relative aperture
focallengths = map(float, range(5,51))
sdistance = 1000.0
bd = 5000.0
aperture = 2.8
fd = open("data/formats/ra-stayput-focallengths.dat", "w")
for fl in focallengths:
  bc = calculate_blur(sdistance, bd, aperture, fl)
  fd.write("%s %s\n" % (fl, bc))

fd.close()

  
